/* Advanced C++ Programming Styles and Idioms */

#include <fstream.h>

class Account {
public:
    Account();
    Account(double penalty, double minimum, const char *name = "Account");
    ~Account();
    virtual void deposit(double) = 0;
    virtual void withdraw(double) = 0;
    virtual void print_statement() = 0;
protected:
    void file_write(const char*);
    void file_read(char*);
protected:
    void _deposit(double);
    void _withdraw(double);
    void _print_statement();
private:
    double penalty;
    double minimum;
    double balance;
    char* name;
    ifstream fpr;
    ofstream fpw;
};

class InterestAccount: virtual public Account {
protected:
    InterestAccount(double penalty, double interest, double minimum,
            char* s = "InterestAccount");
    ~InterestAccount();
public:
    void deposit(double);
    void withdraw(double);
    void print_statement();
    double calculate_interest();
protected:
    void _deposit(double);
    void _withdraw(double);
    void _print_statement();
private:
    double interest;
};

class SavingsAccount: public InterestAccount {
public:
    SavingsAccount(double penalty, double interest, double minimum,
            char* s = "SavingsAccount");
    ~SavingsAccount();
private:
};

class CheckingAccount: virtual public Account {
public:
    CheckingAccount(double penalty, double interest, double minimum,
            char* s = "CheckingAccount");
    void deposit(double);
    void withdraw(double);
    void print_statement();
protected:
    void _deposit(double);
    void _withdraw(double);
    void _print_statement();
private:
};

class InterestCheckingAccount: public CheckingAccount,
                               public InterestAccount {
public:
    InterestCheckingAccount(double penalty, double interest, double minimum,
            char* s = "InterestCheckingAccount");
    void deposit(double);
    void withdraw(double);
    void print_statement();
    double calculate_interest();
protected:
    void _deposit(double);
    void _withdraw(double);
    void _print_statement();
private:
};


