/* Copyright (c) 2003 by Hanbit Media Co. All rights reserved*/
/* Lee, Hae-Young(hae_lee@hanmail.net) */
/* Advanced C++ Programming Styles and Idioms, exercise 5-6 */

#include <iostream.h>

typedef double Frequency;
typedef double Voltage;

// Value class definition
//
class Value {
public:
    enum    Type { T_LPF, T_Data, T_Printer, T_Amplifier };

    Value(Voltage v=0, Frequency w1=0): volts(v), omega1(w1) { }
    
    virtual Value*    evaluate(Value* = 0) { return this; }
    virtual Type      type()    { return T_Data; }
    virtual Frequency f1()      { return omega1; }
    virtual Voltage   v()       { return volts; }

    virtual void print() {
        cout << "Value " << volts << " volts at frequency " << omega1 << endl;
    }
    
private:
    Frequency omega1;
    Voltage   volts;
};

Value Zero(0);
Value *zero = &Zero;

// Filter class definition
//
class Filter: public Value {
public:
    Filter(Frequency w1, Frequency w2 = 0):
        Value(0, w1), omega2(w2), cachedInput(0) {
    }
    virtual Value* evaluate(Value* = 0) = 0;
    virtual Value* operator()(Value*) = 0;

    Frequency f2() { return omega2; }
    Type      type() {
        if (cachedInput) return cachedInput->type();
        else return baseType;
    }

    virtual void print() = 0;

protected:
    Type  myType, baseType;
    Value *cachedInput;
private:
    Frequency omega2;
};

// LPF class definition
//
class LPF: public Filter {
public:
    LPF(Frequency w1): Filter(w1) { baseType = T_LPF; }
    
    Value* evaluate(Value* input=0) {
        Value *f = cachedInput->evaluate(input);
        if (f->f1() < f1()) return f;
        else return zero;
    }
    Value* operator()(Value*);
    void print() {
        if (cachedInput) cout << "COMPOUND" << endl;
        else cout << "LPF(" << f1() << ")" << endl;
    }
};

Value* LPF::operator()(Value* f) {
    switch (f->type()) {
    case T_Data:
        myType = T_Data;
        cachedInput = f;
        return evaluate();
    case T_LPF:
        if (f->f1() > f1()) return this;
        else return f;
    default:
        return f;
    }
};

// Amplifier class definition
//
class Amplifier : public Value {
public:
    Amplifier() { }
  
    Value* evaluate(Value* input, Value* factor);
    Value* operator()(Value*, Value*);

    void print() {
        cout << "AMPLIFIER" << endl;
    }
};
   
Value* Amplifier::evaluate(Value* f, Value* factor) {
   if (f->type() == T_Data) 
        return new Value(f->v()*factor->v(), f->f1());
    else return f;
}

Value* Amplifier::operator ()(Value* f, Value* factor) {
    switch (f->type()) {
    case T_Data:
        return evaluate(f, factor);
    default:
         return f;
    }
}

// Printer class definition
//
class Printer : public Filter {
public:
    Printer() : Filter(0)   {}
    Type   type()           { return T_Printer; }
    Value* evaluate(Value* input=0) { return zero; }
    void   print()          {}
    Value* operator() (Value* v) {
        v->print();
        return 0;
    }
} printerObject;

Printer* printer = &printerObject;

// In the book, object is defined as Value* type. 
// However, Value class does not define any operator(). 
Value* apply(Filter* object, Value* arg) { return (*object)(arg); }
Value* apply(Amplifier* object, Value* arg1, Value* arg2) { return (*object)(arg1, arg2); }

int main() {
    Value*     input  = new Value(100, 1260);
    LPF*       lpf    = new LPF(2000);
    Amplifier* amp    = new Amplifier;
    Value*     factor = new Value(3);

    apply(printer, input);
    apply(printer, apply(lpf, input));
    apply(printer, apply(amp, input, factor));

    delete input;
    delete lpf;
    delete amp;
    delete factor;

    return 0;
}
