/* Copyright (c) 2003 by Hanbit Media Co. All rights reserved*/
/* Lee, Hae-Young(hae_lee@hanmail.net) */
/* Advanced C++ Programming Styles and Idioms, exercise 7-2 */

#include <iostream.h>

const int STACK_SIZE = 10;

template<class T>
class StackInterface{
public:
   virtual T pop()=0;
   virtual void push(T)=0;
   virtual void print()= 0;
};

template<class T> 
class Stack : public StackInterface<T> {
public:
    Stack(int size = STACK_SIZE) { 
        _size  = size; 
        _items = new T[_size]; 
        _sp    = -1; }

    Stack(const Stack& c) { 
        _size  = c._size;
        _items = new T[_size];
        _sp    = c._sp;
        for (int i = 0; i < _sp; i++) 
            _items[i] = c._items[i];
    }

    ~Stack()       { if (_items) delete[] _items; }

    Stack& operator=(const Stack& c) {
        if (_items) delete [] _items;
        _size  = c._size;
        _items = new T[_size];
        _sp    = c._sp;
        for (int i = 0; i < _sp; i++) 
            _items[i] = c._items[i];
    }

    T pop()        { return _items[_sp--]; }
    void push(T t) { _items[++_sp] = t; }
    void print()   { cout << "size = " << _size << endl;
                     cout << "sp   = " << _sp << endl; 
                     cout << "contents = ";
                     for (int i = 0; i <= _sp; i++) cout << _items[i] << "  ";
                     cout << endl; }
private:
    T*  _items;
    int _size, _sp;
};

template<class T>
class CountedStack : public StackInterface<T> {
public:
    CountedStack() : rep (new CountedStackRep) {}
    CountedStack(const CountedStack& c) : rep(c.rep) { rep->count++; }
    CountedStack(const Stack<T>& c) : rep(new CountedStackRep(new Stack<T>(c))) {}
    ~CountedStack() { if (--rep->count <=0) delete rep; }

    CountedStack& operator=(const CountedStack& c) {
        c.rep->count++;
        if (--rep->count <=0) delete rep;
        rep = c.rep; 
        return *this;
    }

    T pop()         { return rep->rep->pop(); }
    void push(T t)  { rep->rep->push(t); }
    void print()    { cout << "reference count = " << rep->count << endl;
                      rep->rep->print(); }
private:
    struct CountedStackRep {
        int         count;
        Stack<T>*   rep;
        CountedStackRep(Stack<T>* s = 0) : count(1) {
            rep = s? s: new Stack<T>;
        }
    } *rep;
};


void clientRoutine(StackInterface<int>* s) {
    s->push(10);
    return;
}

int main() {
    bool debug = true;
    StackInterface<int> *si;
    if (debug == false) {
        si = new Stack<int>;
    } else {
        si = new CountedStack<int>;
    }
    clientRoutine(si);
    si->print();
    delete si;
    return 0;
}
